const googleAuth = require("../utils/googleAuth"); // Import the Google authentication verification logic
const User = require("../models/userModel"); // Import the User model
const jwt = require("jsonwebtoken");
const axios = require("axios");
const path = require("path");
const multer = require("multer");
const fs = require("fs");
const sharp = require("sharp");

const signToken = (id) => {
  return jwt.sign({ id }, process.env.JWT_SECRET, {
    expiresIn: process.env.JWT_EXPIRES_IN,
  });
};
const createSendToken = (user, statusCode, req, res) => {
  const token = signToken(user._id);

  // Ensure the expiration time is a valid number
  const cookieExpireInDays =
    parseInt(process.env.JWT_COOKIE_EXPIRE_IN, 10) || 90; // Default to 90 days if not set

  // Define cookie options
  const cookieOptions = {
    expires: new Date(
      Date.now() + cookieExpireInDays * 24 * 60 * 60 * 1000 // Convert days to milliseconds
    ),
    httpOnly: true, // Prevent client-side script access
    secure: req.secure || req.headers["x-forwarded-proto"] === "https", // Ensure secure cookies in production
    sameSite: "strict", // Helps mitigate CSRF attacks
  };

  // Send cookie
  res.cookie("jwt", token, cookieOptions);

  // Remove sensitive data from the user object
  user.password = undefined;

  // Send response
  res.status(statusCode).json({
    status: true,
    token,
    data: {
      user,
    },
  });
};

// Configure multer for file uploads (same as your original code)
const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    cb(null, "uploads/photos/"); // Set the destination folder for uploaded photos
  },
  filename: (req, file, cb) => {
    const uniqueSuffix = Date.now() + "-" + Math.round(Math.random() * 1e9);
    cb(null, uniqueSuffix + "-" + file.originalname); // Set the file name with a unique suffix
  },
});

// File filter to only accept image files
const fileFilter = (req, file, cb) => {
  if (file.mimetype.startsWith("image/")) {
    cb(null, true);
  } else {
    cb(new Error("Please upload only images."), false);
  }
};

const upload = multer({
  storage,
  fileFilter,
  limits: {
    fileSize: 5 * 1024 * 1024, // Limit file size to 5MB
  },
});

// Helper function to download the image from the URL and save it locally
const saveProfilePic = async (imageUrl) => {
  try {
    // Create a unique filename based on timestamp and random number
    const fileName = `${Date.now()}-${Math.floor(
      Math.random() * 1e8
    )}-${path.basename(imageUrl, path.extname(imageUrl))}.png`;

    // Set the relative path for the file
    const filePath = path.join(__dirname, "..", "uploads", "photos", fileName);

    // Use Axios to download the image and save it locally in the desired directory
    const writer = fs.createWriteStream(filePath);
    const response = await axios({
      url: imageUrl,
      method: "GET",
      responseType: "stream",
    });

    // Pipe the response data to the file
    response.data.pipe(writer);

    return new Promise((resolve, reject) => {
      writer.on("finish", () => resolve(`uploads/photos/${fileName}`)); // Return the relative path for frontend use
      writer.on("error", (err) =>
        reject(new Error("Error downloading profile picture: " + err.message))
      );
    });
  } catch (err) {
    throw new Error("Error downloading profile picture: " + err.message);
  }
};

// Function to authenticate using Google and handle user creation or update
async function authenticateGoogle(req, res) {
  const { access_token } = req.body;

  if (!access_token) {
    return res.status(400).send("Access token is missing");
  }

  try {
    // Step 1: Verify the access token using Google authentication
    const user = await googleAuth.verifyAccessToken(access_token);

    // Step 2: Retrieve additional user information from Google
    const userInfo = await googleAuth.getUserInfoWithAccessToken(access_token);
    const profilePicUrl = userInfo.picture; // Google profile picture URL

    // Step 3: Check if the user already exists in the database
    let existingUser = await User.findOne({ email: userInfo.email });

    // Step 4: If the user exists, update the user
    if (existingUser) {
      existingUser.firstName = userInfo.given_name;
      existingUser.lastName = userInfo.family_name;
      existingUser.emailVerified = userInfo.email_verified;
      existingUser.active = true; // Mark the user as active

      // Download the profile picture if it's not already saved
      if (profilePicUrl && !existingUser.photo) {
        const savedPicPath = await saveProfilePic(profilePicUrl);
        existingUser.photo = savedPicPath;
      }

      // Save updated user info to the database
      await existingUser.save();

      // Generate and send a JWT token using the createSendToken function
      return createSendToken(existingUser, 200, req, res);
    } else {
      // Step 5: If the user doesn't exist, create a new user
      const newUser = new User({
        firstName: userInfo.given_name,
        lastName: userInfo.family_name,
        email: userInfo.email,
        emailVerified: userInfo.email_verified,
        role: "user", // Default to "user"
        active: true, // Mark the user as active
      });

      // Download the profile picture
      const savedPicPath = await saveProfilePic(profilePicUrl);
      newUser.photo = savedPicPath; // Save the path of the saved photo

      await newUser.save(); // Save the new user to the database

      // Generate and send a JWT token using the createSendToken function
      return createSendToken(newUser, 201, req, res);
    }
  } catch (error) {
    console.error("Error verifying token:", error);
    return res.status(400).send("Error verifying token: " + error.message);
  }
}

module.exports = {
  authenticateGoogle,
};
