const Message = require("../models/messageModel");
const { io, getReceiverSocketId,getRoomName,activeRooms } = require("../utils/socket.js");
const mongoose = require("mongoose");
const User = require("../models/userModel"); // User model
const admin = require("../utils/firebaseServices");
const { createAndEmitNotification } = require("../utils/notificationUtils"); // Import the notification handler

const sendImages = async (req, res) => {
  try {
    const senderId = req.user._id;
    const { receiverId } = req.body;

    // Check if files exist
    if (!req.files || req.files.length === 0) {
      return res.status(400).json({ status: false, message: "No images uploaded" });
    }

    // Check if images exceed the limit (max 5)
    if (req.files.length > 5) {
      return res.status(400).json({ status: false, message: "Images must be less than or equal to 5" });
    }

    // Validate IDs
    if (!mongoose.Types.ObjectId.isValid(senderId) || !mongoose.Types.ObjectId.isValid(receiverId)) {
      return res.status(400).json({ status: false, message: "Invalid sender or receiver ID" });
    }

    // Fetch receiver and sender details in parallel
    const [receiver, sender] = await Promise.all([
      User.findById(receiverId).select("fcmToken"),
      User.findById(senderId).select("firstName lastName photo"),
    ]);

    if (!receiver || !sender) {
      return res.status(404).json({ status: false, message: "User not found." });
    }

    // Save the message in the database
    const newMessage = new Message({
      senderId,
      receiverId,
      images: req.files.map((file) => `/uploads/photos/${file.filename}`),
    });

    await newMessage.save();

    // Check if the receiver is in the same chat room
    const roomName = getRoomName(senderId, receiverId);
    if (activeRooms[roomName]?.users.has(receiverId)) {
      await Message.updateOne(
        { _id: newMessage._id },
        { $set: { seen: true } }
      );
      newMessage.seen = true;
    }
    else if (receiver.fcmToken) {
      const fcmPayload = {
        notification: {
          title: "New Images Received",
          body: `${sender.firstName} ${sender.lastName} sent you images.`,
        },
        data: {
          senderId: senderId.toString(),
          receiverId: receiverId.toString(),
          click_action: "FLUTTER_NOTIFICATION_CLICK",
          url: `/message/${newMessage._id}`,
        },
        token: receiver.fcmToken,
      };

      await admin.messaging().sendMulticast(fcmPayload);
    }

    // Emit real-time messages if the receiver is online
    const receiverSocketId = getReceiverSocketId(receiverId);
    if (receiverSocketId) {
      io.to(receiverSocketId).emit("newMessage", {
        _id: newMessage._id,
        senderId,
        receiverId,
        text: "Images sent",
        images: newMessage.images,
      });
    }

    // Send response back to sender
    return res.status(200).json({
      status: true,
      message: "Images sent successfully!",
      data: newMessage,
    });
  } catch (err) {
    console.error("Error in sending images:", err);
    return res.status(500).json({ status: false, message: "Internal server error" });
  }
};



module.exports = { sendImages };
