const axios = require("axios");
const jwt = require("jsonwebtoken");
const User = require("../models/userModel");
const path = require("path");
const multer = require("multer");
const fs = require("fs");
const sharp = require("sharp");

const LINKEDIN_USER_INFO_URL = "https://api.linkedin.com/v2/me";
const LINKEDIN_EMAIL_URL =
  "https://api.linkedin.com/v2/emailAddress?q=members&projection=(elements*(handle~))";
const USERINFO_URL = "https://api.linkedin.com/v2/userinfo";
const AUTHORIZATION_URL = "https://www.linkedin.com/oauth/v2/authorization";
const TOKEN_URL = "https://www.linkedin.com/oauth/v2/accessToken";

const signToken = (id) => {
  return jwt.sign({ id }, process.env.JWT_SECRET, {
    expiresIn: process.env.JWT_EXPIRES_IN,
  });
};

const createSendToken = (user, statusCode, req, res) => {
  const token = signToken(user._id);
  console.log(`The token: ${token}`);

  res.cookie("jwt", token, {
    expires: new Date(Date.now() + 90 * 24 * 60 * 60 * 1000), // 90 days
    httpOnly: true,
    secure: req.secure || req.headers["x-forwarded-proto"] === "https",
    sameSite: "strict",
  });

  user.password = undefined;

  res.status(statusCode).json({
    status: true,
    token,
    data: { user },
  });
};
// Configure multer for file uploads (same as your original code)
const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    const uploadPath = path.join(__dirname, "uploads", "photos"); // Absolute path
    cb(null, uploadPath);
  },
  filename: (req, file, cb) => {
    const uniqueSuffix = Date.now() + "-" + Math.round(Math.random() * 1e9);
    cb(null, uniqueSuffix + "-" + file.originalname); // Add unique suffix to filename
  },
});

// File filter to only accept image files
const fileFilter = (req, file, cb) => {
  if (file.mimetype.startsWith("image/")) {
    cb(null, true);
  } else {
    cb(new Error("Please upload only images."), false);
  }
};

const upload = multer({
  storage,
  fileFilter,
  limits: {
    fileSize: 5 * 1024 * 1024, // Limit file size to 5MB
  },
});
const sanitizeFilename = (filename) => {
  // Remove query parameters and special characters that are not allowed in filenames
  return filename.replace(/[^a-zA-Z0-9\.\-\_]/g, "_").split("?")[0];
};
// Helper function to download the image from the URL and save it locally
const saveProfilePic = async (imageUrl) => {
  try {
    // Create a unique filename based on timestamp and random number
    const fileName = sanitizeFilename(
      `${Date.now()}-${Math.floor(Math.random() * 1e8)}-${path.basename(
        imageUrl,
        path.extname(imageUrl)
      )}.png`
    );
    console.log(fileName);
    // Set the relative path for the file
    const photoDir = path.join(__dirname, "..", "uploads", "photos");

    // Ensure the directory exists
    if (!fs.existsSync(photoDir)) {
      fs.mkdirSync(photoDir, { recursive: true });
    }

    const filePath = path.join(photoDir, fileName);

    // Download image and save it locally
    const writer = fs.createWriteStream(filePath);
    const response = await axios({
      url: imageUrl,
      method: "GET",
      responseType: "stream",
    });

    response.data.pipe(writer);

    return new Promise((resolve, reject) => {
      writer.on("finish", () => resolve(`uploads/photos/${fileName}`)); // Return the relative path for frontend use
      writer.on("error", (err) =>
        reject(new Error("Error downloading profile picture: " + err.message))
      );
    });
  } catch (err) {
    throw new Error("Error downloading profile picture: " + err.message);
  }
};

async function authenticateLinkedIn(req, res) {
  const { access_token } = req.body;
  console.log(access_token);
  if (!access_token) {
    return res
      .status(400)
      .json({ status: false, message: "Access token is missing" });
  }

  try {
    // Fetch user profile data using access_token as a query parameter
    // Fetch user profile
    const profileResponse = await axios.get(USERINFO_URL, {
      headers: { Authorization: `Bearer ${access_token}` },
    });

    const profile = profileResponse.data;
    // console.log(profile);

    // User object
    const user = {
      sub: profile.sub,
      email_verified: profile.email_verified,
      name: `${profile.given_name} ${profile.family_name}`,
      locale: profile.locale,
      given_name: profile.given_name,
      family_name: profile.family_name,
      email: profile.email,
      picture: profile.picture,
    };

    console.log("User Data:", user);
    let existingUser = await User.findOne({ email: profile.email });

    if (existingUser) {
      // Update user if they already exist
      existingUser.name = user.name;
      existingUser.picture = user.picture; // Ensure profile picture is updated
      await existingUser.save();
      return createSendToken(existingUser, 200, req, res);
    } else {
      // Create a new user
      const newUser = new User({
        firstName: user.given_name,
        lastName: user.family_name,
        email: user.email,
        emailVerified: user.email_verified,
        role: "user", // Default to "user"
        active: true, // Mark the user as active
      });
      const profilePicUrl = user.picture; // Google profile picture URL

      const savedPicPath = await saveProfilePic(profilePicUrl);
      newUser.photo = savedPicPath; // Save the path of the saved photo

      await newUser.save(); // Save the new user to the database

      // Generate and send a JWT token using the createSendToken function
      return createSendToken(newUser, 201, req, res);
    }

    // res.json({
    //   status: true,
    //   message: "Authentication successful!",
    //   user: existingUser,
    // });
  } catch (error) {
    console.error(
      "Error in LinkedIn callback:",
      error.response?.data || error.message
    );
    res.status(500).json({
      status: false,
      error: "Failed to complete LinkedIn authentication.",
      details: error.response?.data || error.message,
    });
  }
}

module.exports = {
  authenticateLinkedIn,
};
