const axios = require("axios");
const catchAsync = require("../utils/catchAsync");
const User = require("../models/userModel");

exports.updateCurrentLocation = catchAsync(async (req, res, next) => {
  const { latitude, longitude } = req.body;
  const userId = req.user._id; // Authenticated user ID

  try {
    // Validate latitude and longitude
    if (!latitude || !longitude || isNaN(latitude) || isNaN(longitude)) {
      return res.status(400).json({
        status: false,
        error: "Valid latitude and longitude are required.",
      });
    }

    // Call Google Maps Geocoding API to get place_id
    const googleMapsUrl = `https://maps.googleapis.com/maps/api/geocode/json?latlng=${latitude},${longitude}&key=${process.env.Google_API_Key}`;
    const geocodeResponse = await axios.get(googleMapsUrl);
    const geocodeResults = geocodeResponse.data.results;

    if (!geocodeResults || geocodeResults.length === 0) {
      return res.status(400).json({
        status: false,
        error: "Unable to fetch location details.",
      });
    }

    // Extract place_id from the Geocoding API response
    const placeId = geocodeResults[0].place_id;

    if (!placeId) {
      return res.status(400).json({
        status: false,
        error: "Unable to fetch place_id from the location details.",
      });
    }

    // Find high-rated places within a 500-meter radius
    const radius = 500; // Search within a 500-meter radius
    const nearbySearchUrl = `https://maps.googleapis.com/maps/api/place/nearbysearch/json?location=${latitude},${longitude}&radius=${radius}&key=${process.env.Google_API_Key}`;
    const nearbySearchResponse = await axios.get(nearbySearchUrl);
    const places = nearbySearchResponse.data.results;

    // Find the highest-rated place
    const highestRatedPlace = places.reduce((highest, place) => {
      return place.rating > (highest?.rating || 0) ? place : highest;
    }, null);

    // Save the highest-rated place in the user's data
    const savedPlace = highestRatedPlace
      ? {
          name: highestRatedPlace.name,
          address: highestRatedPlace.vicinity,
          placeId: highestRatedPlace.place_id,
          rating: highestRatedPlace.rating,
          types: highestRatedPlace.types,
          photos: highestRatedPlace.photos
            ? highestRatedPlace.photos.map((photo) => ({
                photoReference: photo.photo_reference,
                url: `https://maps.googleapis.com/maps/api/place/photo?maxwidth=400&photoreference=${photo.photo_reference}&key=${process.env.Google_API_Key}`,
              }))
            : [],
        }
      : null;
     const formattedAddress = savedPlace.name || "Unknown Location";
    await User.findByIdAndUpdate(userId, {
      latitude,
      longitude,
      placeId, // Save place_id
      location: formattedAddress, // Save formatted address
    });

    res.status(200).json({
      status: true,
      message: "Location updated successfully.",
      location: { latitude, longitude, placeId, formattedAddress },
    });
  } catch (error) {
    console.error("Error updating location:", error.message);
    res.status(500).json({
      status: false,
      error: "Something went wrong while updating location.",
    });
  }
});