const User = require("../models/userModel.js");
const Message = require("../models/messageModel.js");
const { io, getReceiverSocketId,getRoomName,activeRooms } = require("../utils/socket.js");
const catchAsync = require("../utils/catchAsync.js");
const mongoose = require("mongoose");
const moment = require("moment");
const { createAndEmitNotification } = require("../utils/notificationUtils.js");
const admin = require("../utils/firebaseServices");
exports.getUsersForSidebar = async (req, res) => {
  try {
    const loggedInUserId = req.user._id;
    console.log(loggedInUserId);
    const filteredUsers = await User.find({
      _id: { $ne: loggedInUserId },
    }).select("-password");

    res.status(200).json(filteredUsers);
  } catch (error) {
    console.error("Error in getUsersForSidebar: ", error.message);
    res.status(500).json({ error: "Internal server error" });
  }
};

/**
 * Get chat messages between the logged-in user and a specific user.
 */
exports.getMessages = async (req, res) => {
  try {
    const { id: userToChatId } = req.params; // ID of the user to chat with
    const myId = req.user._id; // Logged-in user's ID

    // Validate userToChatId
    if (!mongoose.Types.ObjectId.isValid(userToChatId)) {
      return res
        .status(400)
        .json({ status: false, message: "Invalid user ID" });
    }

    const userToChatObjectId = new mongoose.Types.ObjectId(userToChatId);

    // Fetch user details (firstName, lastName, photo, _id)
    const userToChat = await User.findById(userToChatObjectId).select(
      "firstName lastName photo _id"
    );

    if (!userToChat) {
      return res.status(404).json({ status: false, message: "User not found" });
    }

    // ✅ Add online status
    const isUserOnline = !!getReceiverSocketId(userToChatObjectId);
    userToChat._doc.online_status = isUserOnline; // Add online status to user object

    // Fetch messages between the logged-in user and the other user
    const messages = await Message.find({
      $or: [
        { senderId: myId, receiverId: userToChatObjectId },
        { senderId: userToChatObjectId, receiverId: myId },
      ],
    })
      .sort({ createdAt: -1 }) // Sort by time (latest first)
      .populate("senderId", "firstName lastName photo _id")
      .populate("receiverId", "firstName lastName photo _id")
      .lean(); // Convert Mongoose documents to plain objects for modification

    // Format timestamps for each message in HH:mm format
    messages.forEach((message) => {
      const messageTime = moment(message.createdAt);
      message.formattedTime = messageTime.format("HH:mm"); // Format to show time in 24-hour format
    });

    // Get the last message between users
    const lastMessage =
      messages.length > 0 ? messages[messages.length - 1] : null;

    res.status(200).json({
      status: true,
      user: userToChat, // User details with online status
      messages,
      lastMessage,
    });
  } catch (error) {
    console.error("Error in getMessages:", error);
    res.status(500).json({ status: false, error: "Internal server error" });
  }
};


exports.getLastMessages = async (req, res) => {
  try {
    const { id: userToChatId } = req.params; // ID of the user to chat with
    const myId = req.user._id; // Get the logged-in user's ID from JWT

    // Validate userToChatId
    if (!mongoose.Types.ObjectId.isValid(userToChatId)) {
      return res
        .status(400)
        .json({ status: false, message: "Invalid user ID" });
    }

    const userToChatObjectId = new mongoose.Types.ObjectId(userToChatId);

    // Fetch messages between the logged-in user and the other user
    const messages = await Message.find({
      $or: [
        { senderId: myId, receiverId: userToChatObjectId },
        { senderId: userToChatObjectId, receiverId: myId },
      ],
    })
      .sort({ createdAt: 1 }) // Sort by time (oldest first)
      .populate("senderId", "firstName lastName email  recentJob company photo")
      .populate(
        "receiverId",
        "firstName lastName email  recentJob company photo"
      );

    // Get the last message between users
    const lastMessage = await Message.findOne({
      $or: [
        { senderId: myId, receiverId: userToChatObjectId },
        { senderId: userToChatObjectId, receiverId: myId },
      ],
    })
      .sort({ createdAt: -1 }) // Get the most recent message
      .populate("senderId", "firstName lastName email")
      .populate("receiverId", "firstName lastName email");

    // If no messages exist
    if (!messages.length) {
      return res.status(404).json({
        status: false,
        message: "No messages found",
      });
    }

    // Respond with messages and the last message
    res.status(200).json({
      status: true,
      lastMessage, // Adding the last message
    });
  } catch (error) {
    console.error("Error in getMessages:", error);
    res.status(500).json({ status: false, error: "Internal server error" });
  }
};
/**
 * Send a message to a specific user, emit it via Socket.IO, and create a notification.
 */
exports.sendMessage = async (req, res) => {
  try {
    const { text } = req.body;
    const { id: receiverId } = req.params;
    const senderId = req.user?._id || req.cookies.guestId;

    // Validate sender and receiver
    if (!senderId || !receiverId) {
      return res.status(400).json({ error: "Sender and receiver are required." });
    }

    // Fetch receiver and sender data in parallel
    const [receiver, sender] = await Promise.all([
      User.findById(receiverId).select("fcmToken"),
      User.findById(senderId).select("firstName lastName photo"),
    ]);

    if (!receiver || !sender) {
      return res.status(404).json({ error: "User not found." });
    }

    // Create and save the message
    const newMessage = new Message({
      senderId,
      receiverId,
      text,
      seen: false,
    });
    await newMessage.save();

    // Check if the receiver is in the same chat room
    const roomName = getRoomName(senderId, receiverId);
    if (activeRooms[roomName]?.users.has(receiverId)) {
      await Message.updateOne(
        { _id: newMessage._id },
        { $set: { seen: true } }
      );
      newMessage.seen = true;
    } else if (receiver.fcmToken) {
      // Send push notification if the receiver is offline
      const fcmPayload = {
        notification: {
          title: `${sender.firstName} ${sender.lastName}`,
          body: text,
        },
        data: {
          senderId: senderId.toString(),
          receiverId: receiverId.toString(),
          click_action: "FLUTTER_NOTIFICATION_CLICK",
          url: `/messages/${senderId}`,
        },
        token: receiver.fcmToken,
      };
      await admin.messaging().send(fcmPayload);
    }

    // Emit the message to the receiver if they are online
    const receiverSocketId = getReceiverSocketId(receiverId);
    if (receiverSocketId) {
      io.to(receiverSocketId).emit("newMessage", newMessage);
    }

    res.status(201).json(newMessage);
  } catch (error) {
    console.error("Error in sendMessage:", error.message);
    res.status(500).json({ error: "Internal server error" });
  }
};
exports.getMessagedUsers = async (req, res) => {
  try {
    if (!req.user || !req.user._id) {
      return res.status(401).json({ status: false, message: "Unauthorized" });
    }

    const userId = new mongoose.Types.ObjectId(req.user._id);

    // Find distinct users the logged-in user has messaged
    const messages = await Message.find({
      $or: [{ senderId: userId }, { receiverId: userId }],
    }).select("senderId receiverId text createdAt");

    const userIds = new Set();
    const lastMessages = {};

    messages.forEach((msg) => {
      const otherUserId =
        msg.senderId.toString() === userId.toString()
          ? msg.receiverId?.toString()
          : msg.senderId?.toString();

      if (otherUserId) {
        userIds.add(otherUserId);

        // Keep track of the last message for each user
        if (
          !lastMessages[otherUserId] ||
          new Date(msg.createdAt) >
            new Date(lastMessages[otherUserId].createdAt)
        ) {
          lastMessages[otherUserId] = {
            text: msg.text,
            createdAt: msg.createdAt,
          };
        }
      }
    });

    // Fetch user details
    const users = await User.find({ _id: { $in: [...userIds] } }).select(
      "firstName lastName photo"
    );

    // Attach last message to each user
    const usersWithLastMessage = users.map((user) => ({
      _id: user._id,
      firstName: user.firstName,
      lastName: user.lastName,
      photo: user.photo,
      lastMessage: lastMessages[user._id.toString()] || null,
    }));

    return res.status(200).json({
      status: true,
      data: usersWithLastMessage,
    });
  } catch (error) {
    console.error("Error in getMessagedUsers:", error);
    return res
      .status(500)
      .json({ status: false, message: "Internal server error" });
  }
};
// Handle marking a message as seen
exports.seenMessage = async (req, res) => {
  try {
    const receiverId = req.user._id; // Get authenticated user's ID
    const { senderId } = req.body; // Get sender ID from request body

    if (!senderId) {
      return res.status(400).json({
        status: false,
        error: "Sender ID is required.",
      });
    }
const seen =true;
    // Update only unseen messages where the user is the receiver and sender matches
    const result = await Message.updateMany(
      { receiverId, senderId, seen: false },
      { $set: { seen: true } }
    );
 const senderSocketId = getReceiverSocketId(senderId);
    if (senderSocketId) {
      io.to(senderSocketId).emit("messageSeen", {
        senderId,
        receiverId, // Who saw the message
        seen, // Who saw the message
      });
    }
    return res.status(200).json({
      status: true,
      message: `Messages from sender ${senderId} marked as seen`,
      data: {
        modifiedCount: result.modifiedCount, // Number of messages updated
      },
    });
  } catch (error) {
    console.error("Error updating message seen status:", error);
    return res.status(500).json({
      status: false,
      error: "Error updating message seen status",
    });
  }
};



// Handle updating a message
exports.updateMessage = async (req, res) => {
  try {
    const { messageId, newText, userId } = req.body;
    const message = await Message.findById(messageId);

    if (!message) return res.status(404).json({ error: "Message not found" });

    // Only allow sender to update the message
    if (message.senderId.toString() !== userId) {
      return res
        .status(403)
        .json({ error: "Not authorized to update this message" });
    }

    message.text = newText;
    await message.save();

    res.status(200).json({ status: true, messageId, newText });
  } catch (error) {
    console.error("Error updating message:", error);
    res.status(500).json({ error: "Error updating message" });
  }
};

// Handle deleting a message
exports.deleteMessage = async (req, res) => {
  try {
    const { messageId, userId } = req.body;
    const message = await Message.findById(messageId);

    if (!message) return res.status(404).json({ error: "Message not found" });

    // Only allow sender to delete the message
    if (message.senderId.toString() !== userId) {
      return res
        .status(403)
        .json({ error: "Not authorized to delete this message" });
    }

    await Message.findByIdAndDelete(messageId);

    res
      .status(200)
      .json({ status: true, message: "message delete successfully" });
  } catch (error) {
    console.error("Error deleting message:", error);
    res.status(500).json({ error: "Error deleting message" });
  }
};
// Handle typing status
exports.typingStatus = (req, res) => {
  try {
    const { userId, receiverId, typing } = req.body;

    // Emit the typing status to the receiver (this doesn't need to update the database)
    const receiverSocketId = getReceiverSocketId(receiverId);
    if (receiverSocketId) {
      io.to(receiverSocketId).emit("typing", { userId, typing });
    }

    res.status(200).json({ status: true, typingStatus: typing });
  } catch (error) {
    console.error("Error handling typing status:", error);
    res.status(500).json({ error: "Error handling typing status" });
  }
};
