const stripe = require("stripe")(process.env.STRIPE_SECRET_KEY);
const admin = require("firebase-admin"); // Firebase admin for push notifications
const catchAsync = require("../utils/catchAsync");
const User = require("../models/userModel");
const Payment = require("../models/paymentModel"); // Ensure this model exists

// Create Checkout Session
exports.createCheckoutSession = catchAsync(async (req, res, next) => {
  const { credits } = req.body;
  const amount = credits * 0.1867 * 100; // Convert to cents

  const session = await stripe.checkout.sessions.create({
    payment_method_types: ["card"],
    success_url: `${req.protocol}://${req.get(
      "host"
    )}/profile?alert=creditsAdded`,
    cancel_url: `${req.protocol}://${req.get("host")}/buy-credits`,
    customer_email: req.user.email,
    client_reference_id: req.user.id,
    metadata: {
      credits: credits.toString(),
    },
    line_items: [
      {
        price_data: {
          currency: "usd",
          product_data: {
            name: `${credits} Credits Purchase`,
            description: "Add credits to your account",
          },
          unit_amount: amount,
        },
        quantity: 1,
      },
    ],
    mode: "payment",
  });

  res.status(200).json({
    status: "success",
    sessionId: session.id,
  });
});

// Webhook for Checkout Completion
exports.webhookCheckout = (req, res) => {
  const sig = req.headers["stripe-signature"];
  const endpointSecret = process.env.STRIPE_WEBHOOK_SECRET;

  let event;
  try {
    event = stripe.webhooks.constructEvent(req.rawBody, sig, endpointSecret);
  } catch (err) {
    return res.status(400).send(`Webhook Error: ${err.message}`);
  }

  if (event.type === "checkout.session.completed") {
    const session = event.data.object;
    updateUserCredits(session);
  }

  res.status(200).json({ received: true });
};

// Update User Credits and Send Push Notification
const updateUserCredits = async (session) => {
  const userId = session.client_reference_id;
  const credits = 14; // Add 14 credits on successful payment
  const senderId = "System"; // Sender ID can be an admin or system

  const user = await User.findByIdAndUpdate(
    userId,
    { $inc: { credits: credits } }, // Increment credits
    { new: true }
  );

  if (!user) {
    console.error("User not found for credit update.");
    return;
  }

  // Send push notification to user
  if (user.fcmToken) {
    const notificationPayload = {
      notification: {
        title: "Payment Successful 🎉",
        body: `Your payment was successful! You've received ${credits} credits.`,
      },
      data: {
        click_action: "FLUTTER_NOTIFICATION_CLICK",
        credits: credits.toString(),
        senderId: senderId, // System or Admin ID
        receiverId: userId, // The user receiving the notification
      },
      token: user.fcmToken,
    };

    try {
      await admin.messaging().send(notificationPayload);
      console.log("Push notification sent successfully!");
    } catch (error) {
      console.error("Error sending push notification:", error);
    }
  }
};

// Retrieve Payment Details
exports.getPaymentDetails = catchAsync(async (req, res, next) => {
  const { paymentId } = req.params;

  try {
    const user = await User.findById(req.user.id);
    if (!user) {
      return res.status(404).json({ status: "error", message: "User not found" });
    }

    let stripeCustomer;
    if (user.stripeCustomerId) {
      stripeCustomer = await stripe.customers.retrieve(user.stripeCustomerId);
    } else {
      stripeCustomer = await stripe.customers.create({
        email: user.email,
        name: `${user.firstName} ${user.lastName}`,
      });

      user.stripeCustomerId = stripeCustomer.id;
      await user.save();
    }

    const paymentIntent = await stripe.paymentIntents.retrieve(paymentId);
    if (!paymentIntent) {
      return res.status(404).json({ status: "error", message: "Payment not found" });
    }

    const paymentData = {
      paymentId: paymentIntent.id,
      amount: paymentIntent.amount / 100,
      currency: paymentIntent.currency,
      status: paymentIntent.status,
      created: new Date(paymentIntent.created * 1000),
      customerId: stripeCustomer.id,
      user_id: user._id,
    };

    const payment = new Payment(paymentData);
    await payment.save();

    const credits = 14; // Add 14 credits on successful payment
    await User.findByIdAndUpdate(req.user.id, { $inc: { credits: credits } });

    res.status(200).json({
      status: true,
      payment: paymentData,
    });
  } catch (error) {
    res.status(500).json({ status: false, message: error.message });
  }
});

// Retrieve User Payments
exports.getUserPayments = catchAsync(async (req, res, next) => {
  try {
    const user = await User.findById(req.user.id);
    if (!user) {
      return res.status(404).json({ status: "error", message: "User not found" });
    }

    const payments = await Payment.find({ user_id: user._id });

    if (payments.length === 0) {
      return res.status(404).json({ status: "error", message: "No payments found" });
    }

    res.status(200).json({
      status: true,
      payments: payments,
    });
  } catch (error) {
    res.status(500).json({ status: false, message: error.message });
  }
});
