const Message = require("../models/messageModel");
const { io, getReceiverSocketId, getRoomName, activeRooms } = require("../utils/socket.js");
const User = require("../models/userModel"); // User model
const mongoose = require("mongoose");
const { createAndEmitNotification } = require("../utils/notificationUtils");
const admin = require("../utils/firebaseServices");

exports.sendVoiceMessage = async (req, res) => {
  try {
    const senderId = req.user._id;
    const { receiverId } = req.params;

    // Fetch sender and receiver details in parallel
    const [sender, receiver] = await Promise.all([
      User.findById(senderId).select("firstName lastName photo"),
      User.findById(receiverId).select("fcmToken"),
    ]);

    if (!sender || !receiver) {
      return res.status(404).json({ status: false, message: "User not found." });
    }

    // Check if files are uploaded
    if (!req.files || req.files.length === 0) {
      return res.status(400).json({
        status: false,
        message: "No voice messages uploaded",
      });
    }

    // Save all messages in a single database operation
    const messages = req.files.map((file) => ({
      senderId,
      receiverId,
      audio: `/uploads/audio/${file.filename}`,
      seen: false,
    }));

    const savedMessages = await Message.insertMany(messages);

    // Check if the receiver is in the same chat room
    const roomName = getRoomName(senderId, receiverId);
    if (activeRooms[roomName]?.users.has(receiverId)) {
      await Message.updateMany(
        { _id: { $in: savedMessages.map((msg) => msg._id) } },
        { $set: { seen: true } }
      );
    } else if (receiver.fcmToken) {
      const fcmPayload = {
        notification: {
          title: "New Voice Message",
          body: `${sender.firstName} ${sender.lastName} sent you a voice message.`,
        },
        data: {
          senderId: senderId.toString(),
          receiverId: receiverId.toString(),
          audioUrl: `/uploads/audio/${req.files[0].filename}`,
          url: `/messages/${senderId}`,
          click_action: "FLUTTER_NOTIFICATION_CLICK",
        },
        token: receiver.fcmToken, // Use `token` for single device
      };

      // Send notification to a single device
      await admin.messaging().send(fcmPayload);
    }

    // Emit real-time messages if the receiver is online
    const receiverSocketId = getReceiverSocketId(receiverId);
    if (receiverSocketId) {
      io.to(receiverSocketId).emit("newMessage", savedMessages);
    }

    return res.status(200).json({
      status: true,
      message: "Voice messages sent successfully!",
      data: savedMessages,
    });
  } catch (err) {
    console.error("Error sending voice messages:", err);
    return res.status(500).json({ status: false, message: "Internal server error" });
  }
};