const mongoose = require("mongoose");
const validator = require("validator");
const bcrypt = require("bcryptjs");

const connectionModel = require("./connectionModel"); // Import the connection model

const userSchema = new mongoose.Schema(
  {
    firstName: {
      type: String,
    },
    lastName: {
      type: String,
    },
    othername: {
      type: String,
    },
    email: {
      type: String,
      unique: true,
      lowercase: true,
      validate: [validator.isEmail, "Please provide a valid email"],
    },

    photo: {
      type: String,
    },
    role: {
      type: String,
      enum: ["user", "admin"],
      default: "user",
    },
    password: {
      type: String,
      select: false,
    },
    passwordConfirm: {
      type: String,
    },
    bio: {
      type: String,
    },
    location: {
      type: String,
    },
    placeId: {
      type: String,
    },
      longitude: {
      type: Number,

    },
      latitude: {
      type: Number,
    },
  
    recentJob: {
      type: String,
    },
    phone: {
      type: String,
      trim: true,
    },
    country: {
      type: String,
      trim: true,
    },
    company: {
      type: String,
    },
    stripeCustomerId: String,
    credits: { type: Number, default: 3 },
    socialLinks: {
      linkedin: { type: String, default: "" },
      github: { type: String, default: "" },
      twitter: { type: String, default: "" },
      facebook: { type: String, default: "" },
      website: { type: String, default: "" },
      instagram: { type: String, default: "" },
      whatsapp: { type: String, default: "" },
      addLink: { type: String, default: "" },
    },
    interests: { type: [String], default: [] },
    otps: [
      {
        otp: {
          type: String,
        },
        otpExpires: {
          type: Date,
        },
      },
    ],
    passwordChangedAt: Date,
    passwordResetExpires: Date,
    active: {
      type: Boolean,
      default: false,
    },
    connections: [
      {
        type: mongoose.Schema.Types.ObjectId,
        ref: "User", // Change this from "Connection" to "User"
      },
    ],
    lockTime: {
      type: Number, // Lock time in milliseconds
      default: 0,
    },
    lastAccessTime: {
      type: Date,
      default: Date.now,
    },
    isLocked: {
      type: Boolean,
      default: false,
    },
    isVisible: {
      type: Boolean,
      default: true, // Profile is visible by default
    },
    visibilityEndTime: {
      type: Date, // When the profile visibility should end
      default: null,
    },
    totalConnections: {
      type: Number,
      default: 0, // Default to zero connections
    },
    fcmToken: {
      type: String,
      default: null, // Store the FCM token for push notifications
    },
  },
  { timestamps: true }
);

// Pre-save hooks, password hashing, etc. (Same as in the original schema)

userSchema.pre("save", async function (next) {
  if (!this.isModified("password")) return next();
  this.password = await bcrypt.hash(this.password, 12);
  this.passwordConfirm = undefined;
  next();
});
// Middleware to set status to "" if not already set
userSchema.pre("save", function (next) {
  if (this.isNew && (this.status === null || this.status === undefined)) {
    this.status = ""; // Set status to empty string if it's not defined
  }
  next();
});

userSchema.pre("save", function (next) {
  if (!this.isModified("password") || this.isNew) return next();
  this.passwordChangedAt = Date.now() - 1000;
  next();
});

userSchema.methods.correctPassword = async function (
  candidatePassword,
  userPassword
) {
  return await bcrypt.compare(candidatePassword, userPassword);
};

userSchema.methods.calculateProfileCompletion = function () {
  const requiredFields = [
    "firstName",
    "lastName",
    "email",
    "recentJob",
    "company",
    "bio",
    "country",
    "phone",
    "interests",
  ];

  let completedFields = 0;
  requiredFields.forEach((field) => {
    if (field === "interests") {
      if (this[field] && this[field].length > 0) {
        completedFields++;
      }
    } else if (this[field] && this[field].trim() !== "") {
      completedFields++;
    }
  });

  const completionPercentage = (completedFields / requiredFields.length) * 100;
  return Math.round(completionPercentage);
};

userSchema.methods.changedPasswordAfter = function (JWTTimestamp) {
  if (this.passwordChangedAt) {
    const changedTimestamp = parseInt(
      this.passwordChangedAt.getTime() / 1000,
      10
    );
    return JWTTimestamp < changedTimestamp;
  }
  return false;
};

const User = mongoose.model("User", userSchema);

module.exports = User;