const path = require("path");
const express = require("express");
const morgan = require("morgan");
const helmet = require("helmet");
const mongoSanitize = require("express-mongo-sanitize");
const xss = require("xss-clean");
const cookieParser = require("cookie-parser");
const bodyParser = require("body-parser");
const compression = require("compression");
const cors = require("cors");
const session = require("express-session");
const passport = require("passport");
const dotenv = require("dotenv");
const cron = require("node-cron");
const mongoose = require("mongoose");
const { app, server } = require("./utils/socket");

const User = require("./models/userModel");
const AppError = require("./utils/appError");
const globalErrorHandler = require("./controllers/errorController");
const userRouter = require("./routes/userRoutes");
const authRoutes = require("./routes/authRoutes");
const interestRoutes = require("./routes/interestRoutes");
const connectionRoutes = require("./routes/connectionRoutes");
const lockerRoutes = require("./routes/lockRoutes");
const googleRoutes = require("./routes/googleAuthRoutes");
const linkedinRoutes = require("./routes/linkedinAuthRoutes");
const messageRoutes = require("./routes/messageRoutes");
const paymentRoutes = require("./routes/paymentRoutes");
const locationRoutes = require("./routes/locationRoutes");
const imgSocketRoutes = require("./routes/imgSocketRoutes");
const voiceMessageRoutes = require("./routes/voiceMessageRoutes");
const notificationRoutes = require("./routes/notificationRoutes");
const pushNotificationRoutes = require("./routes/pushNotificationRoutes");

// Load environment variables
dotenv.config({ path: "./config.env" });

// MongoDB connection setup
const DB = process.env.DATABASE.replace(
  "<db_password>",
  process.env.DATABASE_PASSWORD
);

// Connect to MongoDB
mongoose
  .connect(DB, {
    // useNewUrlParser: true, // Use new MongoDB URL parser
    // Use the new server discovery and monitoring engine
  })
  .then(() => console.log("DB Connection Successful!"))
  .catch((err) => {
    console.error("MongoDB connection error:", err.message);
    process.exit(1); // Exit process on connection error
  });

app.set("view engine", "pug");
app.set("views", path.join(__dirname, "views"));

// Middleware to parse JSON requests
app.use(bodyParser.json());

// 1) GLOBAL MIDDLEWARES
app.enable("trust proxy");
app.use(
  cors({
    origin: "",
    credentials: true,
  })
);

// CORS Configuration
const corsOptions = {
  origin: process.env.CLIENT_URL || "http://localhost:5173", // Replace with actual frontend URL in production
  methods: "GET,POST,PUT,DELETE",
  allowedHeaders: "Content-Type,Authorization",
  credentials: true, // Allow cookies to be sent in cross-origin requests (if needed)
};
app.use(cors(corsOptions));

// Serving static files
app.use(express.static(path.join(__dirname, "public")));

// Serve static files from the "uploads" folder (for uploaded images)
app.use("/uploads", express.static(path.join(__dirname, "uploads")));

// Set security HTTP headers
app.use(helmet());

// Development logging
if (process.env.NODE_ENV === "development") {
  app.use(morgan("dev"));
}

// Body parser, reading data from the body into req.body
app.use(express.json({ limit: "10kb" }));
app.use(express.urlencoded({ extended: true, limit: "10kb" }));
app.use(cookieParser());

// Data sanitization against NoSQL query injection
app.use(mongoSanitize());

// Data sanitization against XSS
app.use(xss());

// Compression
app.use(compression());

app.use(
  session({
    secret: process.env.SESSION_SECRET || "your-secret-key",
    resave: false,
    saveUninitialized: false,
    cookie: {
      secure: process.env.NODE_ENV === "production",
      httpOnly: true,
      maxAge: 1000 * 60 * 60 * 24, // 1 day
    },
  })
);

// Cron job to check if any profile's visibility time has expired
cron.schedule("* * * * *", async () => {
  try {
    // Get users whose visibility time has passed and the profile is still hidden
    const users = await User.find({
      visibilityEndTime: { $lte: Date.now() }, // Check if current time is greater than visibilityEndTime
      isVisible: false, // Profile is currently hidden
    });

    users.forEach(async (user) => {
      // Make the profile visible again after the lock time has passed
      user.isVisible = true;
      user.visibilityEndTime = null; // Reset the visibility end time
      await user.save();

      console.log(`User ${user._id}'s profile is now visible again.`);
    });
  } catch (err) {
    console.error("Error checking profile visibility:", err);
  }
});

// Initialize passport for authentication
app.use(passport.initialize());
app.use(passport.session());

// Test middleware
app.use((req, res, next) => {
  req.requestTime = new Date().toISOString();
  next();
});

// 2) ROUTES

app.get("/", (req, res) => {
  res.status(200).send({
    status: true,
    message: "Welcome to App",
  });
});

// Mount routes
app.use("/api/auth/google", googleRoutes);
app.use("/api/auth/linkedin", linkedinRoutes);
app.use("/api/auth", authRoutes); // Use the separated auth routes
// Endpoint to verify the ID token sent from the Android app

app.use("/api/v1/users", userRouter);
app.use("/api/v1/interests", interestRoutes);
app.use("/api/v1/connections", connectionRoutes);
app.use("/api/v1/lock", lockerRoutes);
app.use("/api/messages", messageRoutes);
app.use("/api/v1/img", imgSocketRoutes);
app.use("/api/v1/location", locationRoutes);
app.use("/api/v1/payments", paymentRoutes);
app.use("/api/v1/messages", messageRoutes);
app.use("/api/v1/voice", voiceMessageRoutes);
app.use("/api/v1/notification", notificationRoutes);
app.use("/api/v1/push", pushNotificationRoutes);

// Protected dashboard route
app.get("/dashboard", (req, res) => {
  if (!req.isAuthenticated()) {
    return res.redirect("/auth/linkedin");
  }
  res.send(`<h1>Welcome ${req.user.displayName}</h1>`);
});

// Handle undefined routes
app.all("*", (req, res, next) => {
  next(new AppError(`Can't find ${req.originalUrl} on this server!`, 404));
});

// Global error handling middleware
app.use(globalErrorHandler);

const port = process.env.PORT || 3000;

server.listen(port, () => {
  console.log(`Server is running on port ${port}...`);
});

// Handle uncaught exceptions
process.on("uncaughtException", (err) => {
  console.error("UNCAUGHT EXCEPTION! 😨 Shutting down...");
  console.error(err.name, err.message);
  process.exit(1);
});

// Handle unhandled promise rejections
process.on("unhandledRejection", (err) => {
  console.error("UNHANDLED REJECTION! 🔥 Shutting down...");
  console.error(err.name, err.message);
  server.close(() => {
    process.exit(1);
  });
});

// Handle SIGTERM (graceful shutdown)
process.on("SIGTERM", () => {
  console.log("👋 SIGTERM RECEIVED. Shutting down gracefully...");
  mongoose.connection.close(() => {
    console.log("MongoDB connection closed.");
    server.close(() => {
      console.log("🔥 Process terminated!");
    });
  });
});
