const { io, getReceiverSocketId } = require("./socket"); // Import io instance from socket.js
const Notification = require("../models/notificationModel");

/**
 * Function to create and emit notification to a user
 * @param {Object} io - The Socket.IO instance
 * @param {string} userId - The user ID to send the notification to
 * @param {string} message - The notification message
 * @param {string} type - Type of notification (e.g., "message", "alert")
 * @param {string} link - URL to redirect the user (can be a link to the message or user profile)
 */
const createAndEmitNotification = async (
  io,
  userId,
  message,
  type,
  link,
  senderId
) => {
  try {
    // Validate type to ensure it's one of the allowed values
    const validTypes = ["connection", "like", "comment", "message", "other"];
    if (!validTypes.includes(type)) {
      throw new Error("Invalid notification type.");
    }

    // Create a new notification for the user
    const newNotification = new Notification({
      user: userId, // User who will receive the notification
      message: message,
      type: type, // Ensure this is one of the valid types
      link: link,
      senderId: senderId,
    });
    // Save the notification to the database
    await newNotification.save();
    console.log("Notifiction is send");

    // Check if the user is online
    const receiverSocketId = getReceiverSocketId(userId);

    if (receiverSocketId) {
      // If the user is online, send the real-time notification
      io.to(receiverSocketId).emit("New Notification", {
        message: newNotification.message,
        createdAt: newNotification.createdAt,
        link: newNotification.link,
      });

      // Update the notification as delivered
      newNotification.isPending = false;
      await newNotification.save();

      console.log(`Notification sent to user ${userId}`);
    } else {
      // If the user is offline, the notification will remain saved and pending
      console.log(`User ${userId} is offline. Notification saved.`);
    }
  } catch (error) {
    console.error("Error creating and emitting notification:", error.message);
    // Optionally, log more details about the error, including the user and message
    console.error(
      `Failed to send notification to user ${userId} with message: ${message}`
    );
  }
};

/**
 * Function to send all pending notifications to a user when they come online
 * @param {Object} io - The Socket.IO instance
 * @param {string} userId - The user ID who has come online
 */
const sendPendingNotifications = async (io, userId) => {
  try {
    // Fetch all pending notifications for the user
    const notifications = await Notification.find({
      user: userId,
      isPending: true,
    });

    if (notifications.length > 0) {
      // Emit all pending notifications to the user
      notifications.forEach(async (notification) => {
        io.to(userId.toString()).emit("New Notification", {
          message: notification.message,
          createdAt: notification.createdAt,
          link: notification.link,
        });

        // After sending the notification, mark it as delivered
        notification.isPending = false;
        await notification.save();
      });

      console.log(
        `Sent ${notifications.length} pending notifications to user ${userId}`
      );
    } else {
      console.log(`No pending notifications for user ${userId}`);
    }
  } catch (error) {
    console.error("Error sending pending notifications:", error.message);
  }
};

module.exports = {
  createAndEmitNotification,
  sendPendingNotifications,
};
